@@ -131,326 +131,55 @@ setMethod("dimSupp", signature = "multiFunData",
 setMethod("dimSupp", signature = "irregFunData",
           function(object){length(dim(as.array(object@argvals[[1]])))})
 
-#### Plot ####
 
-#' Plotting univariate functional data
+#### ApproxNA ####
+
+#' Approximate missing values for funData objects
 #' 
-#' This function plots observations of univariate functional data on their 
-#' domain.
+#' This function approximates missing values for \code{funData} objects based on
+#' the \link[zoo]{na.approx} interpolation method from the package \pkg{zoo}.
 #' 
-#' If some observations contain missing values (coded via \code{NA}), the 
-#' functions can be interpolated using the option \code{plotNA = TRUE}. This 
-#' option relies on the \code{\link[zoo]{na.approx}} function in package 
-#' \code{\link[zoo]{zoo}} and is currently implemented for one-dimensional 
-#' functions only.
-#' 
-#' @section Warning: The function is currently implemented only for functional
-#'   data with one- and two-dimensional domains.
+#' @section Warning: This function requires the package \pkg{zoo} to be 
+#'   installed, otherwise it will throw a warning.
 #'   
-#' @param x An object of class \code{funData}.
-#' @param y Missing.
-#' @param obs A vector of numerics giving the observations to plot. Defaults to 
-#'   all observations in \code{x}. For two-dimensional functions (images) 
-#'   \code{obs} must have length 1.
-#' @param type The type of plot. Defaults to \code{"l"} (line plot). See 
-#'   \code{\link[graphics]{plot}} for details.
-#' @param lty The line type. Defaults to \code{1} (solid line). See 
-#'   \code{\link[graphics]{par}} for details.
-#' @param lwd The line width. Defaults to \code{1}. See \code{\link[graphics]{par}} for
-#'   details.
-#' @param col The color of the functions. If not supplied (\code{NULL}, default 
-#'   value), one-dimensional functions are plotted in the 
-#'   \code{\link[grDevices]{rainbow}} palette and two-dimensional functions are 
-#'   plotted using \code{\link[fields]{tim.colors}} from package 
-#'   \code{\link[fields]{fields-package}}.
-#' @param xlab,ylab The titles for x- and y-axis. Defaults to \code{"argvals"} for the 
-#'   x-axis and no title for the y-axis. See \code{\link[graphics]{plot}} for 
-#'   details.
-#' @param legend Logical. If \code{TRUE}, a color legend is plotted for 
-#'   two-dimensional functions (images). Defaults to \code{TRUE}.
-#' @param plotNA Logical. If \code{TRUE}, missing values are interpolated (only 
-#'   for one-dimensional functions). Defaults to \code{FALSE}. See Details.
-#' @param add Logical. If \code{TRUE}, add to current plot (only for 
-#'   one-dimensional functions). Defaults to \code{FALSE}.
-#' @param ... Additional arguments to \code{\link[graphics]{matplot} } 
-#'   (one-dimensional functions) or \code{\link[fields]{image.plot}}/
-#'   \code{\link[graphics]{image}} (two-dimensional functions).
+#' @param object An object of  class \code{funData} with missing values (coded 
+#'   by \code{NA}).
 #'   
-#' @seealso \code{\linkS4class{funData}}, \code{\link[graphics]{matplot}}, 
-#'   \code{\link[fields]{image.plot}}, \code{\link[graphics]{image}}
+#' @return A \code{funData} object where missing values have been imputed.
 #'   
-#' @importFrom grDevices rainbow
-#' @importFrom graphics matplot image
+#' @export approxNA
 #'   
 #' @examples
+#' # Simulate some data
+#' f <- simFunData(N = 10, M = 8, eVal = "linear", eFun = "Poly", argvals = seq(0, 1, 0.01))$simData
+#' 
+#' # Sparsify, i.e. generate artificial missings in the data
+#' fSparse <- sparsify(f, minObs = 10, maxObs = 50)
+#' 
+#' # plot
 #' oldpar <- par(no.readonly = TRUE)
-#' 
-#' # One-dimensional
-#' argvals <- seq(0,2*pi,0.01)
-#' object <- funData(argvals,
-#'                    outer(seq(0.75, 1.25, length.out = 11), sin(argvals)))
-#' 
-#' plot(object, main = "One-dimensional functional data")
-#' 
-#' # Two-dimensional
-#' X <- array(0, dim = c(2, length(argvals), length(argvals)))
-#' X[1,,] <- outer(argvals, argvals, function(x,y){sin((x-pi)^2 + (y-pi)^2)})
-#' X[2,,] <- outer(argvals, argvals, function(x,y){sin(2*x*pi) * cos(2*y*pi)})
-#' object2D <- funData(list(argvals, argvals), X)
-#' 
-#' plot(object2D, main = "Two-dimensional functional data (obs 1)", obs = 1)
-#' plot(object2D, main = "Two-dimensional functional data (obs 2)", obs = 2)
-#' \dontrun{plot(object2D, main = "Two-dimensional functional data")} # must specify obs!
-#' 
-#' ### More examples ###
-#' par(mfrow = c(1,1))
-#' 
-#' # using plotNA
-#' if(requireNamespace("zoo", quietly = TRUE))
-#' {
-#' objectMissing <- funData(1:5, rbind(c(1, NA, 5, 4, 3), c(10, 9, NA, NA, 6)))
-#' par(mfrow = c(1,2))
-#' plot(objectMissing, type = "b", pch = 20, main = "plotNA = FALSE") # the default
-#' plot(objectMissing, type = "b", pch = 20, plotNA = TRUE, main = "plotNA = TRUE") # requires zoo
-#' }
-#' 
-#' # Changing colors
-#' plot(object, main = "1D functional data in grey", col = "grey")
-#' plot(object, main = "1D functional data in heat.colors", col = heat.colors(nObs(object)))
-#' 
-#' plot(object2D, main = "2D functional data in topo.colors", obs = 1, col = topo.colors(64))
-
-#' par(oldpar)
-plot.funData <- function(x, y, obs = 1:nObs(x), type = "l", lty = 1, lwd = 1,
-                         col = NULL, xlab = "argvals", ylab = "", legend = TRUE,
-                         plotNA = FALSE, add = FALSE, ...)
-{
-  if(dimSupp(x) > 2)
-    stop("plot is implemented only for functional data with one- or two-dimensional domain")
-  
-  if(dimSupp(x) == 1)
-  {
-    # set default color
-    if(is.null(col))
-      col <-  grDevices::rainbow(length(obs))
-    
-    if(plotNA) # interpolate NA values
-    {
-      # require zoo
-      if (requireNamespace("zoo", quietly = TRUE))
-      {
-        graphics::matplot(x = x@argvals[[1]], y = zoo::na.approx(t(x@X[obs,, drop = FALSE])), type = "l", lty = lty,  lwd = lwd, col = col, xlab = xlab, ylab = ylab, ...)
-        
-        add = TRUE # add the standard plot
-      }
-      else
-        warning("Package zoo needed for interpolating missing values in plot for funData. Ignoring plotNA = TRUE.")
-    }
-    
-    graphics::matplot(x = x@argvals[[1]], y = t(x@X[obs,, drop = FALSE]), type = type, lty = lty,  lwd = lwd, col = col, xlab = xlab, ylab = ylab, add = add, ...)
-  }
-  if(dimSupp(x) == 2)
-  {
-    if(length(obs) > 1)
-      stop("plot: specify one observation for plotting")
-    
-    if(add == TRUE)
-      stop("plot: add = TRUE not implemented for images")
-    
-    # set default color
-    if(is.null(col))
-      col <-  fields::tim.colors(64)
-    
-    if(legend == TRUE)
-    {
-      fields::image.plot(x = x@argvals[[1]], y = x@argvals[[2]], z = x@X[obs, ,], lty = lty, xlab = xlab, ylab = ylab, col = col, ...)
-    }
-    else
-    {
-      graphics::image(x = x@argvals[[1]], y = x@argvals[[2]], z = x@X[obs, ,], lty = lty, xlab = xlab, ylab = ylab, col = col, ...)
-    }
-    
-    
-  }
-}
-
-
-#' Plotting multivariate functional data
-#' 
-#' This function plots observations of multivariate functional data on their 
-#' domain. The graphic device is split in a number of subplots (specified by 
-#' \code{dim}) via \code{mfrow} (\code{\link[graphics]{par}}) and the univariate
-#' elements are plotted using \code{plot}.
-#' 
-#' @section Warning: The function is currently implemented only for functional 
-#'   data with one- and two-dimensional domains.
-#'   
-#' @param x An object of class \code{multiFunData}.
-#' @param y Missing.
-#' @param obs A vector of numerics giving the observations to plot. Defaults to 
-#'   all observations in \code{x}. For two-dimensional functions (images) 
-#'   \code{obs} must have length 1.
-#' @param dim The dimensions to plot. Defaults to \code{length(x)}, i.e. all
-#'   functions in \code{x} are plotted.
-#' @param par.plot Graphic parameters to be passed to the plotting regions. The 
-#'   option \code{mfrow} is ignored. Defaults to \code{FALSE}. See 
-#'   \code{\link[graphics]{par}} for details.
-#' @param main A string vector, giving the title of the plot. Can have the same 
-#'   length as \code{dim} (different titles for each dimension) or length 
-#'   \code{1} (one title for all dimensions). Defaults to \code{NULL}.
-#' @param xlab,ylab The titles for x- and y-axis. Defaults to \code{"argvals"} for the 
-#'   x-axis and no title for the y-axis for all elements. Can be supplied as a 
-#'   vector of the same length as \code{x} (one x-/y-lab for each element) or a 
-#'   single string that is applied for all elements. See 
-#'   \code{\link[graphics]{plot}} for details.
-#' @param ... Additional arguments to \code{plot}.
-#'   
-#' @seealso \code{\linkS4class{funData}}, \code{\linkS4class{multiFunData}}, 
-#'   \code{\link{plot.funData}}
-#'   
-#' @examples
-#' oldpar <- par(no.readonly = TRUE)
-#' argvals <- seq(0, 2*pi, 0.1)
-#' 
-#' # One-dimensional elements
-#' f1 <- funData(argvals, outer(seq(0.75, 1.25, length.out = 11), sin(argvals)))
-#' f2 <- funData(argvals, outer(seq(0.75, 1.25, length.out = 11), cos(argvals)))
-#' 
-#' m1 <- multiFunData(f1, f2)
-#' plot(m1, main = c("1st element", "2nd element")) # different titles
-#' plot(m1, main = "Multivariate Functional Data") # one title for all
-#' 
-#' # Mixed-dimensional elements
-#' X <- array(0, dim = c(11, length(argvals), length(argvals)))
-#' X[1,,] <- outer(argvals, argvals, function(x,y){sin((x-pi)^2 + (y-pi)^2)})
-#' g <- funData(list(argvals, argvals), X)
-#' 
-#' m2 <- multiFunData(f1, g)
-#' # different titles and labels
-#' plot(m2, main = c("1st element", "2nd element"), obs = 1,
-#'      xlab = c("xlab1", "xlab2"), 
-#'      ylab = "one ylab for all")
-#' # one title for all
-#' plot(m2, main = "Multivariate Functional Data", obs = 1) 
-#' 
-#' \dontrun{plot(m2, main = c("1st element", "2nd element")) # must specify obs!}
+#' par(mfrow = c(1,3))
+#' plot(f, main = "Original Data") 
+#' plot(fSparse, main = "Sparse Data")
+#' plot(approxNA(fSparse), main = "Reconstructed Data")
+#' # faster with plot(fSparse, plotNA = TRUE, main = "Reconstructed Data")
 #' 
 #' par(oldpar)
-plot.multiFunData <- function(x, y, obs = 1:nObs(x), dim = 1:length(x), par.plot = NULL, main = NULL, 
-                              xlab = "argvals", ylab = "", ...){
-  
-  if(length(xlab) == 1)
-    xlab <- rep(xlab, length(x))
-  
-  if(length(ylab) == 1)
-    ylab <- rep(ylab, length(x))
-  
-  
-  # if no par.plot specified: get graphics parameters
-  if(is.null(par.plot))
-  {
-    oldPar <- par(no.readonly = TRUE)
-  }  else
-  {
-    par(par.plot)
-  }
-  
-  # split screen
-  par(mfrow = c(1,length(dim)))
-  
-  
-  if(!is.null(main) & (length(main) == 1))
-    main <- rep(main, length(dim))
-  
-  
-  # plot the univariate functions
-  for(i in dim)
-    plot(x[[i]], obs = obs, main = main[i], xlab = xlab[i], ylab = ylab[i], ...)
-  
-  
-  # if no par.plot specified: reset graphics parameters
-  if(is.null(par.plot))
-    par(oldPar)
-}
+setGeneric("approxNA", function(object) {standardGeneric("approxNA")})
 
-#' Plotting irregular functional data
-#' 
-#' This function plots observations of irregular functional data on their 
-#' domain.
-#' 
-#' @param x An object of class \code{irregFunData}.
-#' @param y Missing.
-#' @param obs A vector of numerics giving the observations to plot. Defaults to 
-#'   all observations in \code{x}.
-#' @param type The type of plot. Defaults to \code{"b"} (line and point plot).
-#'   See \code{\link[graphics]{plot}} for details.
-#' @param pch The point type. Defaults to \code{20} (solid small circles). See 
-#'   \code{\link[graphics]{par}} for details.
-#' @param col The color of the functions. Defaults to the 
-#'   \code{\link[grDevices]{rainbow}} palette.
-#' @param xlab,ylab The titles for x- and y-axis. Defaults to \code{"argvals"}
-#'   for the x-axis and no title for the y-axis. See
-#'   \code{\link[graphics]{plot}} for details.
-#' @param xlim,ylim The limits for x- and y-axis. Defaults to the total range of
-#'   the data that is to plot. See \code{\link[graphics]{plot}} for details.
-#' @param add Logical. If \code{TRUE}, add to current plot (only for 
-#'   one-dimensional functions). Defaults to \code{FALSE}.
-#' @param ... Additional arguments to \code{\link[graphics]{plot}}.
-#'   
-#' @seealso \code{\link{plot.funData}}, \code{\linkS4class{irregFunData}},
-#'   \code{\link[graphics]{plot}}
-#'   
-#'  @importFrom grDevices rainbow
-#' @importFrom graphics points
-#'   
-#' @examples
-#' oldpar <- par(no.readonly = TRUE)
-#' 
-#' # Generate data
-#' argvals <- seq(0,2*pi,0.01)
-#' ind <- replicate(5, sort(sample(1:length(argvals), sample(5:10,1))))
-#' object <- irregFunData(argvals = lapply(ind, function(i){argvals[i]}),
-#'                   X = lapply(ind, function(i){sample(1:10,1) / 10 * argvals[i]^2}))
-#' 
-#' plot(object, main = "Irregular functional data")
-#' 
-#' par(oldpar)
-plot.irregFunData <- function(x, y, obs = 1:nObs(x), type = "b", pch = 20,
-                              col = grDevices::rainbow(nObs(x)), xlab = "argvals", ylab = "",
-                              xlim = range(x@argvals[obs]), ylim = range(x@X[obs]),
-                              add = FALSE, ...)
-{
-  if(length(col) < nObs(x))
-    col <- rep(col, nObs(x))
-  
-  if(add == FALSE) # plot new window
-  {
-    plot(x = NULL, y = NULL, type = "n", xlim = xlim, ylim = ylim,  xlab = xlab, ylab = ylab, ...)
-  }
-  
-  for(i in obs)
-    graphics::points(x = x@argvals[[i]], y = x@X[[i]], type = type, pch = pch, col = col[i], ...)
-  
-}
-
-#' @rdname plot.funData
+#' approxNA for funData objects
 #'
-#' @exportMethod plot
-setMethod("plot", signature = signature(x = "funData", y = "missing"),
-          function(x,y,...){plot.funData(x,y,...)})
-
-#' @rdname plot.multiFunData
-#'
-#' @exportMethod plot
-setMethod("plot", signature =  signature(x = "multiFunData", y = "missing"),
-          function(x,y,...){plot.multiFunData(x,y,...)})
-
-#' @rdname plot.irregFunData
-#'
-#' @exportMethod plot
-setMethod("plot", signature = signature(x = "irregFunData", y = "missing"),
-          function(x,y,...){plot.irregFunData(x,y,...)})
-
+#' @keywords internal
+setMethod("approxNA", signature = "funData",
+          function(object){
+            # require zoo
+            if (requireNamespace("zoo", quietly = TRUE))
+            {
+              return(funData(object@argvals, t(zoo::na.approx(t(object@X), na.rm = FALSE))))
+            }
+            else
+              warning("Package zoo needed for interpolating missing values in plot for funData. Ignoring plotNA = TRUE.")
+          })
 
 #### Arith ####
 
@@ -706,6 +435,77 @@ setMethod("Arith", signature = c(e1 = "funData", e2 = "irregFunData"),
             irregFunData(argvals = e2@argvals, X = sapply(1:nObs(e2), function(i){do.call(f, list(e2@X[[i]], e1@X[i,e1@argvals[[1]] %in% e2@argvals[[i]]]))}, simplify = FALSE))
           })
 
+
+
+#' Mathematical operations for functional data objects
+#' 
+#' These functions allow to apply mathematical operations (such as \eqn{exp(),
+#' log(), sin(), cos()} or \eqn{abs()} to functional data objects based on
+#' \code{\link[methods]{Math}}.  The operations are made pointwise for each
+#' observation.
+#'   
+#' @param x An nbject of class \code{funData}, \code{irregFunData} or
+#'   \code{multiFunData}.
+#'   
+#' @return An object of the same functional data class as \code{x}.
+#'   
+#' @seealso \code{\linkS4class{funData}}, \code{\linkS4class{irregFunData}},
+#'   \code{\linkS4class{multiFunData}}, \link[methods]{Math}
+#'   
+#' @name Math.funData
+#'   
+#' @examples
+#' oldpar <- par(no.readonly = TRUE)
+#' par(mfrow = c(1,2))
+#' 
+#' # simulate a funData object on 0..1 with 10 observations
+#' argvals <- seq(0, 1, 0.01)
+#' f <- simFunData(argvals = argvals, N = 10, 
+#'                 M = 5, eFunType = "Fourier", eValType = "linear")$simData
+#' 
+#' ### FunData
+#' plot(f, main = "Original data")
+#' plot(abs(f), main = "Absolute values")
+#' 
+#' ### Irregular
+#' # create an irrgFunData object by sparsifying f
+#' i <- as.irregFunData(sparsify(f, minObs = 5, maxObs = 10))
+#' 
+#' plot(i, main = "Sparse data")
+#' plot(cumsum(i), main = "'cumsum' of sparse data")
+#' 
+#' ### Multivariate
+#' m <- multiFunData(f, -1*f)
+#' plot(m, main = "Multivariate Data")
+#' plot(exp(m), main = "Exponential")
+#' 
+#' par(oldpar)
+NULL
+#' @rdname Math.funData
+setMethod("Math", signature = c(x = "funData"),
+          function(x){
+            funData(x@argvals, methods::callGeneric(x@X))
+          })
+
+#' @rdname Math.funData
+setMethod("Math", signature = c(x = "multiFunData"),
+          function(x){
+            m <- vector("list", length(x))
+            for( i in 1:length(x))
+              m[[i]] <- methods::callGeneric(x[[i]])
+            multiFunData(m)
+          })
+          
+#' @rdname Math.funData
+setMethod("Math", signature = c(x = "irregFunData"),
+          function(x){
+            generic <- methods::getGeneric(as.character(sys.call())[[1]], mustFind = TRUE, where = environment())
+            f <- environment(generic)$.Generic # helper function (callGeneric not applicable in lapply)
+            
+            irregFunData(argvals = x@argvals, X = lapply(x@X, f))
+          })
+
+
 #### nObs ####
 
 #' Get the number of observations
@@ -1253,7 +1053,7 @@ norm.funData <- function(object, squared, obs, method, weight)
 setMethod("norm", signature = "funData",
           function(object, squared = TRUE, obs = 1:nObs(object), method = "trapezoidal", weight = 1){
             norm.funData(object, squared, obs, method, weight)
-            })
+          })
 
 #' Calculate the norm for multivariate functional data
 #'
@@ -1309,6 +1109,138 @@ setMethod("norm", signature = "irregFunData",
             norm.irregFunData(object, squared, obs, method, weight, fullDom)
           })
 
+#### Scalar product ####
+
+#' Calculate the scalar product for functional data objects
+#' 
+#' This function calculates the scalar product between two objects of the class 
+#' \code{\linkS4class{funData}}, \code{\linkS4class{irregFunData}} and 
+#' \code{\linkS4class{multiFunData}}. For univariate functions \eqn{f,g} on a
+#' domain \eqn{\mathcal{T}}{\calT}, the scalar product is defined as 
+#' \deqn{\int_\mathcal{T} f(t) g(t) \mathrm{d}t}{\int_\calT f(t) g(t) dt} and 
+#' for multivariate functions \eqn{f,g} on domains \eqn{\mathcal{T}_1, \ldots, 
+#' \mathcal{T}_p}{\calT_1,\ldots,\calT_p}, it is defined as \deqn{\sum_{j = 1}^p
+#' \int_{\mathcal{T}_j} f^{(j)}(t) g^{(j)}(t) \mathrm{d}t.}{\sum_{j = 1}^p 
+#' \int_\calT_j f^{(j)}(t) g^{(j)}(t) dt.} As seen in the formula, the objects 
+#' must be defined on the same domain. The scalar product is calculated pairwise
+#' for all observations, thus the objects must also have the same number of 
+#' observations or one object may have only one observation (for which the 
+#' scalar product is calculated with all observations of the other object)). 
+#' Objects of the classes \code{\link{funData}} and \code{\link{irregFunData}} 
+#' can be combined, see \code{\link{integrate}} for details.
+#' 
+#' For \code{\linkS4class{multiFunData}} one cann pass optional vector
+#' \code{weight} for calculating a weighted scalar product. This vector must
+#' have the same number of elements as the \code{\link{multiFunData}} objects
+#' and have to be non-negative with at least one weight that is different from
+#' 0. Defaults to \code{1} for each element. See also \code{\link{norm}}.
+#' 
+#' @param object1,object2 Two objects of class\code{\link{funData}}, 
+#'   \code{\link{irregFunData}} or \code{\link{multiFunData}}, for that the 
+#'   scalar product is to be calculated.
+#' @param ... Additional parameters passed to \code{\link{integrate}}. For
+#'   \code{\linkS4class{multiFunData}} objects, one can also pass a
+#'   \code{weight} argument. See Details.
+#'   
+#' @return A vector of length \code{nObs(object1)} (or \code{nObs(object2)}, if 
+#'   \code{object1} has only one observation), containing the pairwise scalar 
+#'   product for each observation.
+#'   
+#' @seealso \code{\link{integrate}}, \code{\link{norm}},
+#'   
+#' @export scalarProduct
+#'   
+#' @examples 
+#' # create two funData objectw with 5 observations on [0,1]
+#' f <- simFunData(N = 5, M = 7, eValType = "linear",
+#'                 eFunType = "Fourier", argvals = seq(0,1,0.01))$simData
+#' g <- simFunData(N = 5, M = 4, eValType = "linear",
+#'                 eFunType = "Poly", argvals = seq(0,1,0.01))$simData
+#'                 
+#' # calculate the scalar product
+#' scalarProduct(f,g)
+#' 
+#' # the scalar product of an object with itself equals the squared norm
+#' all.equal(scalarProduct(f,f), norm(f, squared = TRUE))
+#' 
+#' # This works of course also for multiFunData objects...
+#' m <- multiFunData(f,g)
+#' all.equal(scalarProduct(m,m), norm(m, squared = TRUE))
+#' 
+#' # ...and for irregFunData objects
+#' i <- as.irregFunData(sparsify(f, minObs = 5, maxObs = 10))
+#' all.equal(scalarProduct(i,i), norm(i, squared = TRUE))
+#' 
+#' # Scalar product between funData and irregFunData objects
+#' scalarProduct(i,f)
+#' 
+#' # Weighted scalar product for multiFunData objects
+#' scalarProduct(m,m, weight = c(1,2))
+setGeneric("scalarProduct", function(object1, object2, ...) {standardGeneric("scalarProduct")})
+
+#' Generic method for scalar products, based on integrate
+.scalarProduct <-  function(object1, object2, ...){
+  return(integrate(object1 * object2, ...))
+}
+
+#' Scalar product for functional data
+#'
+#' @seealso \code{\link{integrate}}, \code{\link{norm}}.
+#'
+#' @keywords internal
+setMethod("scalarProduct", signature = c("funData", "funData"),
+         .scalarProduct)
+
+#' Scalar product for multivariate functional data
+#'
+#' @seealso \code{\link{integrate}}, \code{\link{norm}}.
+#'
+#' @keywords internal
+setMethod("scalarProduct", signature = c("multiFunData", "multiFunData"),
+          function(object1, object2, weight = rep(1, length(object1)),...)
+          {
+            if(length(object1) != length(object2))
+              stop("multiFunData objects must have the same number of elements.")
+            
+            if(length(weight) != length(object1))
+              stop("Weight vector must have the same number of elements as the multiFunData objects.")
+            
+            if(any(weight < 0))
+              stop("Weights must be non-negative.")
+            
+            if(all(weight == 0))
+              stop("At least one weighting factor must be different from 0.")
+            
+            if(length(object1) == 1)
+              return(.scalarProduct(object1,object2,...)*weight)
+            # else: more than one element
+            return(as.numeric(mapply(funData:::.scalarProduct, object1, object2, MoreArgs = list(...), SIMPLIFY = "array") %*% weight))
+          })
+
+#' Scalar product for irregular functional data
+#'
+#' @seealso \code{\link{integrate}}, \code{\link{norm}}.
+#'
+#' @keywords internal
+setMethod("scalarProduct", signature = c("irregFunData", "irregFunData"),
+          .scalarProduct)
+
+#' Scalar product for irregular and functional data
+#'
+#' @seealso \code{\link{integrate}}, \code{\link{norm}}.
+#'
+#' @keywords internal
+setMethod("scalarProduct", signature = c("funData", "irregFunData"),
+          .scalarProduct)
+
+#' Scalar product for irregular and functional data
+#'
+#' @seealso \code{\link{integrate}}, \code{\link{norm}}.
+#'
+#' @keywords internal
+setMethod("scalarProduct", signature = c("irregFunData", "funData"),
+          .scalarProduct)
+
 
 #### get/set ####
 
@@ -1411,7 +1343,7 @@ setMethod("getArgvals", signature = "funData",
 #'
 #' @keywords internal
 setMethod("getArgvals", signature = "multiFunData",
-          function(object){sapply(object, getArgvals)})
+          function(object){lapply(object, getArgvals)})
 
 #' Get argvals slot for irregular functional data objects
 #'
@@ -1879,6 +1811,43 @@ setMethod("meanFunction", signature = c("irregFunData", "ANY"),
 
 #### tensorProduct ####
 
+#' Function to expand integers to a grid of indices
+#' 
+#' This function takes an arbitrary number \eqn{K} of integer values \eqn{n_1,\ldots, n_K} and 
+#' creates a data frame with all combinations from \code{1:}\eqn{n_k}, where the first column 
+#' (taking values from 1 to \eqn{n_1}) varies slowest and the last column (())taking values from 1 
+#' to \eqn{n_K}) varies fastest. Internally, this function depends on \code{\link{expand.grid}}
+#' 
+#' @param ... An arbitrary number of integer values.
+#'   
+#' @return A dataframe with the same number of columns as integers supplied and containing all 
+#'   combinations of indices from 1 to the given integers. If no number is supplied, the function
+#'   returns \code{NULL}.
+#'   
+#' @keywords internal
+#'   
+#' @examples 
+#' # For two integers
+#' funData:::expand.int(2,5) # first column varies slowest
+#' 
+#' # For three integers
+#' funData:::expand.int(2,3,4)
+expand.int <- function(...)
+{
+  dots <- list(...)
+  p <- length(dots)
+  
+  if(p == 0)
+    return(NULL)
+  else
+  {
+    res <- expand.grid(lapply(dots[p:1], function(x){1:x}))[,p:1]
+    names(res) <- rev(names(res))
+  }  
+  
+    return(res)
+}
+
 #' Tensor product for univariate functions on one-dimensional domains
 #' 
 #' This function calculates tensor product functions for up to three objects of
@@ -1927,7 +1896,7 @@ setGeneric("tensorProduct", function(...) {standardGeneric("tensorProduct")})
 
 #' Tensor product for functional data
 #'
-#' @seealso \code{\link{meanFunction}}
+#' @seealso \code{\link{tensorProduct}}
 #'
 #' @keywords internal
 setMethod("tensorProduct", signature = c("funData"),
@@ -1941,7 +1910,8 @@ setMethod("tensorProduct", signature = c("funData"),
             if(any(sapply(l, dimSupp) != 1))
               stop("tensorProduct is defined only for funData objects on one-dimensional domains!")
             
-            g <- expand.grid(lapply(l, function(f){1:nObs(f)}))
+            # expand grid of indices: first varies slowest, last varies fastest
+            g <- do.call(expand.int, lapply(l, nObs))
             
             if(length(l) == 2)
             {
