@@ -78,14 +78,25 @@ setGeneric("sparsify", function(funDataObject, minObs, maxObs) {standardGeneric(
 #' @keywords internal
 setMethod("sparsify", signature = "funData",
           function(funDataObject, minObs, maxObs){
+            if(maxObs > nObsPoints(funDataObject))
+              stop("Sparsification: 'maxObs' must not exceed the maximal number of observations")
+
+            if(minObs < 1)
+              stop("Sparsification: 'minObs' must be a positive integer!")
+            
+            if(maxObs < minObs)
+              stop("Sparsification: 'minObs' must be smaller or equal to 'maxObs'.")
+            
             sparseData <- funDataObject
-            
-            n <- length(sparseData@argvals[[1]]) # total number of observation points
-            
+
             for(i in 1:nObs(sparseData)) # for all observed functions
             {
-              Ni <- sample(minObs:maxObs, 1) # number of observation points
-              notNA <- sample(n, Ni) # sample observation points
+              if(minObs == maxObs)
+                Ni <- minObs
+              else
+                Ni <- sample(minObs:maxObs, 1) # number of observation points
+              
+              notNA <- sample(nObsPoints(funDataObject), Ni) # sample observation points
               sparseData@X[i, -notNA] <- NA  # set all other values to NA
             }
             return(sparseData)})
@@ -261,7 +272,7 @@ efFourier <- function(argvals, M, linear = FALSE)
   
   if(linear) # overwrite Phi[M, ], add linear function and orthonormalize (Gram-Schmidt)
   {
-    if(!all.equal(range(argvals) , c(0,1)))
+    if(any(range(argvals) != c(0,1)))
       stop("efFourier, option linear: not yet implemented for argvals != [0,1]!")
     
     # orthogonalize (exact function)
@@ -337,8 +348,8 @@ efWiener <- function(argvals, M)
 #'   \code{\link{simMultiFunData}}
 #'   
 #' @export eFun
-#'   
-#'  @examples
+#'    
+#' @examples
 #' oldPar <- par(no.readonly = TRUE)
 #' 
 #' argvals <- seq(0,1,0.01)
@@ -420,53 +431,49 @@ eVal <- function(M, type)
 
 
 #' Simulate univariate functional data
-#'
-#' This functions simulates (univariate) functional data \eqn{f_1,\ldots, f_N} based on a truncated
-#' Karhunen-Loeve representation: \deqn{f_i(t) = \sum_{m = 1}^M \xi_{i,m}
-#' \phi_m(t).} The eigenfunctions (basis functions) \eqn{\phi_m(t)} are generated
-#' using \code{\link{eFun}}, the scores \eqn{\xi_{i,m}} are simulated
-#' independently from a normal distribution with zero mean and decreasing
-#' variance based on the \code{\link{eVal}} function.
-#'
-#' @param argvals A numeric vector, containing the observation points (a fine grid
-#'   on a real interval) of the functional data that is to be simulated.
-#' @param M An integer, giving the number of unvariate basis functions to use.
-#'   See Details.
-#' @param eFunType A character string specifying the type of univariate
-#'   orthonormal basis functions to use. See \code{\link{eFun}} for details.
-#' @param ignoreDeg A vector of integers, specifying the degrees to ignore when
-#'   generating the univariate orthonormal bases. Defaults to \code{NULL}. See
-#'   \code{\link{eFun}} for details.
-#' @param eValType A character string, specifying the type of
-#'   eigenvalues/variances used for the generation of the simulated functions
-#'   based on the truncated Karhunen-Loeve representation. See
+#' 
+#' This functions simulates (univariate) functional data \eqn{f_1,\ldots, f_N} based on a truncated 
+#' Karhunen-Loeve representation: \deqn{f_i(t) = \sum_{m = 1}^M \xi_{i,m} \phi_m(t).} on one- or
+#' higher-dimensional domains. The eigenfunctions (basis functions) \eqn{\phi_m(t)} are generated 
+#' using \code{\link{eFun}}, the scores \eqn{\xi_{i,m}} are simulated independently from a normal
+#' distribution with zero mean and decreasing variance based on the \code{\link{eVal}} function. For
+#' higher-dimensional domains, the eigenfunctions are constructed as tensors of marginal orthonormal
+#' function systems.
+#' 
+#' @param argvals A numeric vector, containing the observation points (a fine grid on a real
+#'   interval) of the functional data that is to be simulated or a list of the marginal observation points.
+#' @param M An integer, giving the number of unvariate basis functions to use. For higher-dimensional data, \code{M} is a vector with the marginal number of eigenfunctions. See Details.
+#' @param eFunType A character string specifying the type of univariate orthonormal basis functions
+#'   to use. For data on higher-dimensional domains, \code{eFunType} can be a vector, specifying the marginal type of eigenfunctions to use in the tensor product. See \code{\link{eFun}} for details.
+#' @param ignoreDeg A vector of integers, specifying the degrees to ignore when generating the
+#'   univariate orthonormal bases. Defaults to \code{NULL}. For higher-dimensional data, \code{ignoreDeg} can be supplied as list with vectors for each marginal. See \code{\link{eFun}} for details.
+#' @param eValType A character string, specifying the type of eigenvalues/variances used for the
+#'   generation of the simulated functions based on the truncated Karhunen-Loeve representation. See
 #'   \code{\link{eVal}} for details.
-#' @param N An integer, specifying the number of multivariate functions to be
-#'   generated.
-#'
-#' @return \item{simData}{A \code{\linkS4class{funData}} object with \code{N}
-#'   observations, representing the simulated functional data.}
-#'   \item{trueFuns}{A \code{\linkS4class{funData}} object with \code{M}
-#'   observations, representing the true eigenfunction basis used for simulating
-#'   the data.} \item{trueVals}{A vector of numerics, representing the true
-#'   eigenvalues used for simulating the data.}
-#'
-#' @seealso \code{\linkS4class{funData}}, \code{\link{eFun}}, \code{\link{eVal}},
+#' @param N An integer, specifying the number of multivariate functions to be generated.
+#'   
+#' @return \item{simData}{A \code{\linkS4class{funData}} object with \code{N} observations,
+#'   representing the simulated functional data.} \item{trueFuns}{A \code{\linkS4class{funData}}
+#'   object with \code{M} observations, representing the true eigenfunction basis used for
+#'   simulating the data.} \item{trueVals}{A vector of numerics, representing the true eigenvalues
+#'   used for simulating the data.}
+#'   
+#' @seealso \code{\linkS4class{funData}}, \code{\link{eFun}}, \code{\link{eVal}}, 
 #'   \code{\link{addError}}, \code{\link{sparsify}}
-#'
+#'   
 #' @importFrom stats rnorm
-#'
+#'   
 #' @export simFunData
-#'
+#'   
 #' @examples
 #' oldPar <- par(no.readonly = TRUE)
-#'
+#' 
 #' # Use Legendre polynomials as eigenfunctions and a linear eigenvalue decrease
 #' test <- simFunData(seq(0,1,0.01), M = 10, eFunType = "Poly", eValType = "linear", N = 10)
-#'
+#' 
 #' plot(test$trueFuns, main = "True Eigenfunctions")
 #' plot(test$simData, main = "Simulated Data")
-#'
+#' 
 #' # The use of ignoreDeg for eFunType = "PolyHigh"
 #' test <- simFunData(seq(0,1,0.01), M = 4, eFunType = "Poly", eValType = "linear", N = 10)
 #' test_noConst <-  simFunData(seq(0,1,0.01), M = 4, eFunType = "PolyHigh",
@@ -475,30 +482,80 @@ eVal <- function(M, type)
 #'                              ignoreDeg = 2, eValType = "linear", N = 10)
 #' test_noBoth <-  simFunData(seq(0,1,0.01), M = 4, eFunType = "PolyHigh",
 #'                            ignoreDeg = 1:2, eValType = "linear", N = 10)
-#'
+#' 
 #' par(mfrow = c(2,2))
 #' plot(test$trueFuns, main = "Standard polynomial basis (M = 4)")
 #' plot(test_noConst$trueFuns, main = "No constant basis function")
 #' plot(test_noLinear$trueFuns, main = "No linear basis function")
 #' plot(test_noBoth$trueFuns, main = "Neither linear nor constant basis function")
-#'
+#' 
+#' # Higher-dimensional domains
+#' simImages <- simFunData(argvals = list(seq(0,1,0.01), seq(-pi/2, pi/2, 0.02)), 
+#'              M = c(5,4), eFunType = c("Wiener","Fourier"), eValType = "linear", N = 4)
+#' for(i in 1:4) 
+#'    plot(simImages$simData, obs = i, main = paste("Observation", i))
+#'                
 #' par(oldPar)
 simFunData <- function(argvals, M, eFunType, ignoreDeg = NULL, eValType, N)
 {
-  # generate eigenfunctions
-  trueFuns <-  eFun(argvals, M, ignoreDeg = ignoreDeg, type = eFunType)
+  ### transform argvals to list, if necessary
+  if(!is.list(argvals))
+    argvals <- list(argvals)
+
+  ### check consistency of input data
+  p <- length(argvals)
   
-  # generate eigenvalues and scores
-  trueVals <- eVal(M, eValType)
-  scores <- t(replicate(N, stats::rnorm(M, sd = sqrt(eVal(M, eValType)))))
+  if(length(M) != p)
+  { 
+    if(length(M) == 1)
+    {
+      warning("Simulation of tensor product data. The value of M will be used for all dimensions.")
+      M <- rep(M, p)
+    }  
+    else
+      stop("M must have the same length as argvals or 1.")
+  }
   
-  # truncated Karhunen-Loeve representation
-  simData <- funData(argvals, scores %*% trueFuns@X)
+  if(length(eFunType) != p)
+  { 
+    if(length(eFunType) == 1)
+    {
+      warning("Simulation of tensor product data. The value of eFunType will be used for all dimensions.")
+      eFunType <- rep(eFunType, p)
+    }  
+    else
+      stop("eFunType must have the same length as argvals or 1.")
+  }
+  
+  ### generate eigenvalues and scores
+  trueVals <- eVal(prod(M), eValType)
+  scores <- t(replicate(N, stats::rnorm(prod(M), sd = sqrt(trueVals))))
+  
+  ### calculate eigenfunctions
+  if(p == 1) # one-dimensional domain
+  {
+    trueFuns <- eFun(argvals = argvals[[1]], M = M, ignoreDeg = ignoreDeg, type = eFunType)
+    resX <- scores %*% trueFuns@X
+  }  
+  else # tensor product of marginal eigenfunctions
+  {
+    if(is.null(ignoreDeg))
+      ignoreDeg <- vector("list", length(M))
+    
+    trueFuns <- do.call(tensorProduct, mapply(eFun, argvals = argvals, M = M, ignoreDeg = ignoreDeg, type = eFunType))
+    
+    tmp <- trueFuns@X
+    dim(tmp) <- c(prod(M), prod(sapply(argvals, length)))
+    resX <- scores %*% tmp
+    dim(resX) <- c(N, sapply(argvals, length))
+  } 
+    
+  ### truncated Karhunen-Loeve representation
+  simData <- funData(argvals, resX)
   
   return(list(simData = simData,
               trueFuns = trueFuns,
               trueVals = trueVals))
-  
 }
 
 
@@ -748,7 +805,7 @@ simMultiWeight <- function(argvals, M, eFunType, ignoreDeg = NULL, eValType, N)
   
   if(p > 1)
   {
-    if(do.call(all.equal, lapply(M, prod)))
+    if(isTRUE(do.call(all.equal, lapply(M, prod))))
     {
       Mtotal <- prod(M[[1]])
     }
@@ -770,29 +827,11 @@ simMultiWeight <- function(argvals, M, eFunType, ignoreDeg = NULL, eValType, N)
   for(j in 1:p)
   {
     if(dimsSupp[j] == 1) # one-dimensional
-    {
-      basis[[j]] <- weight[j] * eFun(argvals[[j]][[1]], M = M[[j]], ignoreDeg = ignoreDeg[[j]], type = eFunType[[j]])@X
-    }
+      basis[[j]] <- weight[j] * eFun(argvals[[j]][[1]], M = M[[j]], ignoreDeg = ignoreDeg[[j]], type = eFunType[[j]])
     else # dimsSupp[j] == 2, i.e. two-dimensional
-    {
-      # image basis
-      y1 <- eFun(argvals[[j]][[1]], M = M[[j]][1], ignoreDeg = ignoreDeg[[j]][[1]], type = eFunType[[j]][1])@X
-      y2 <- eFun(argvals[[j]][[2]], M = M[[j]][2], ignoreDeg = ignoreDeg[[j]][[2]], type = eFunType[[j]][2])@X
-      
-      basis[[j]] <- array(0, c(prod(M[[j]]), length(argvals[[j]][[1]]), length(argvals[[j]][[2]])))
-      
-      for(l in 1:M[[j]][1])
-      {
-        for(k in 1:M[[j]][2])
-          basis[[j]][(l-1) * M[[j]][2] + k,,] <- weight[j] * y1[l,] %o% y2[k,]
-      }
-    }
+      basis[[j]]  <- weight[j] * tensorProduct(eFun(argvals[[j]][[1]], M = M[[j]][1], ignoreDeg = ignoreDeg[[j]][[1]], type = eFunType[[j]][1]),
+                                               eFun(argvals[[j]][[2]], M = M[[j]][2], ignoreDeg = ignoreDeg[[j]][[2]], type = eFunType[[j]][2]))
   }
   
-  trueFuns <- vector("list", p)
-  
-  for(j in 1:p)
-    trueFuns[[j]] <- funData(argvals[[j]], basis[[j]])
-  
-  return(multiFunData(trueFuns))
+  return(multiFunData(basis))
 }
